@@ -7,15 +7,20 @@ import sys
 from unittest import mock
 
 import asyncio
-from asyncio import events
 from asyncio.proactor_events import BaseProactorEventLoop
 from asyncio.proactor_events import _ProactorSocketTransport
 from asyncio.proactor_events import _ProactorWritePipeTransport
 from asyncio.proactor_events import _ProactorDuplexPipeTransport
+from asyncio.proactor_events import _ProactorDatagramTransport
 from test import support
+from test.support import socket_helper
 from test.test_asyncio import utils as test_utils
 
 
+def tearDownModule():
+    asyncio.set_event_loop_policy(None)
+
+
 def close_transport(transport):
     # Don't call transport.close() because the event loop and the IOCP proactor
     # are mocked
@@ -43,7 +48,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         return transport
 
     def test_ctor(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         tr = self.socket_transport(waiter=fut)
         test_utils.run_briefly(self.loop)
         self.assertIsNone(fut.result())
@@ -58,7 +63,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         self.assertFalse(self.protocol.eof_received.called)
 
     def test_loop_reading_data(self):
-        res = asyncio.Future(loop=self.loop)
+        res = self.loop.create_future()
         res.set_result(b'data')
 
         tr = self.socket_transport()
@@ -67,8 +72,9 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         self.loop._proactor.recv.assert_called_with(self.sock, 32768)
         self.protocol.data_received.assert_called_with(b'data')
 
+    @unittest.skipIf(sys.flags.optimize, "Assertions are disabled in optimized mode")
     def test_loop_reading_no_data(self):
-        res = asyncio.Future(loop=self.loop)
+        res = self.loop.create_future()
         res.set_result(b'')
 
         tr = self.socket_transport()
@@ -178,7 +184,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         m_log.warning.assert_called_with('socket.send() raised exception.')
 
     def test_loop_writing_stop(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         fut.set_result(b'data')
 
         tr = self.socket_transport()
@@ -187,7 +193,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         self.assertIsNone(tr._write_fut)
 
     def test_loop_writing_closing(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         fut.set_result(1)
 
         tr = self.socket_transport()
@@ -256,7 +262,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
     def test_loop_writing_force_close(self):
         exc_handler = mock.Mock()
         self.loop.set_exception_handler(exc_handler)
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         fut.set_result(1)
         self.proactor.send.return_value = fut
 
@@ -299,7 +305,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
 
     def test_write_eof_buffer(self):
         tr = self.socket_transport()
-        f = asyncio.Future(loop=self.loop)
+        f = self.loop.create_future()
         tr._loop._proactor.send.return_value = f
         tr.write(b'data')
         tr.write_eof()
@@ -323,7 +329,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
 
     def test_write_eof_buffer_write_pipe(self):
         tr = _ProactorWritePipeTransport(self.loop, self.sock, self.protocol)
-        f = asyncio.Future(loop=self.loop)
+        f = self.loop.create_future()
         tr._loop._proactor.send.return_value = f
         tr.write(b'data')
         tr.write_eof()
@@ -348,7 +354,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         tr = self.socket_transport()
         futures = []
         for msg in [b'data1', b'data2', b'data3', b'data4', b'data5', b'']:
-            f = asyncio.Future(loop=self.loop)
+            f = self.loop.create_future()
             f.set_result(msg)
             futures.append(f)
 
@@ -402,7 +408,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         tr = self.pause_writing_transport(high=4)
 
         # write a large chunk, must pause writing
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         self.loop._proactor.send.return_value = fut
         tr.write(b'large data')
         self.loop._run_once()
@@ -418,7 +424,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         tr = self.pause_writing_transport(high=4)
 
         # first short write, the buffer is not full (3 <= 4)
-        fut1 = asyncio.Future(loop=self.loop)
+        fut1 = self.loop.create_future()
         self.loop._proactor.send.return_value = fut1
         tr.write(b'123')
         self.loop._run_once()
@@ -435,7 +441,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         tr = self.pause_writing_transport(high=4)
 
         # first short write, the buffer is not full (1 <= 4)
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         self.loop._proactor.send.return_value = fut
         tr.write(b'1')
         self.loop._run_once()
@@ -459,7 +465,7 @@ class ProactorSocketTransportTests(test_utils.TestCase):
 
         # write a large chunk which completes immediately,
         # it should not pause writing
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         fut.set_result(None)
         self.loop._proactor.send.return_value = fut
         tr.write(b'very large data')
@@ -468,257 +474,206 @@ class ProactorSocketTransportTests(test_utils.TestCase):
         self.assertFalse(self.protocol.pause_writing.called)
 
 
-@unittest.skip('FIXME: bpo-33694: these tests are too close '
-               'to the implementation and should be refactored or removed')
-class ProactorSocketTransportBufferedProtoTests(test_utils.TestCase):
+class ProactorDatagramTransportTests(test_utils.TestCase):
 
     def setUp(self):
         super().setUp()
         self.loop = self.new_test_loop()
-        self.addCleanup(self.loop.close)
         self.proactor = mock.Mock()
         self.loop._proactor = self.proactor
+        self.protocol = test_utils.make_test_protocol(asyncio.DatagramProtocol)
+        self.sock = mock.Mock(spec_set=socket.socket)
+        self.sock.fileno.return_value = 7
 
-        self.protocol = test_utils.make_test_protocol(asyncio.BufferedProtocol)
-        self.buf = bytearray(1)
-        self.protocol.get_buffer.side_effect = lambda hint: self.buf
-
-        self.sock = mock.Mock(socket.socket)
-
-    def socket_transport(self, waiter=None):
-        transport = _ProactorSocketTransport(self.loop, self.sock,
-                                             self.protocol, waiter=waiter)
+    def datagram_transport(self, address=None):
+        self.sock.getpeername.side_effect = None if address else OSError
+        transport = _ProactorDatagramTransport(self.loop, self.sock,
+                                               self.protocol,
+                                               address=address)
         self.addCleanup(close_transport, transport)
         return transport
 
-    def test_ctor(self):
-        fut = asyncio.Future(loop=self.loop)
-        tr = self.socket_transport(waiter=fut)
-        test_utils.run_briefly(self.loop)
-        self.assertIsNone(fut.result())
-        self.protocol.connection_made(tr)
-        self.proactor.recv_into.assert_called_with(self.sock, self.buf)
+    def test_sendto(self):
+        data = b'data'
+        transport = self.datagram_transport()
+        transport.sendto(data, ('0.0.0.0', 1234))
+        self.assertTrue(self.proactor.sendto.called)
+        self.proactor.sendto.assert_called_with(
+            self.sock, data, addr=('0.0.0.0', 1234))
 
-    def test_loop_reading(self):
-        tr = self.socket_transport()
-        tr._loop_reading()
-        self.loop._proactor.recv_into.assert_called_with(self.sock, self.buf)
-        self.assertTrue(self.protocol.get_buffer.called)
-        self.assertFalse(self.protocol.buffer_updated.called)
-        self.assertFalse(self.protocol.eof_received.called)
+    def test_sendto_bytearray(self):
+        data = bytearray(b'data')
+        transport = self.datagram_transport()
+        transport.sendto(data, ('0.0.0.0', 1234))
+        self.assertTrue(self.proactor.sendto.called)
+        self.proactor.sendto.assert_called_with(
+            self.sock, b'data', addr=('0.0.0.0', 1234))
 
-    def test_get_buffer_error(self):
-        transport = self.socket_transport()
+    def test_sendto_memoryview(self):
+        data = memoryview(b'data')
+        transport = self.datagram_transport()
+        transport.sendto(data, ('0.0.0.0', 1234))
+        self.assertTrue(self.proactor.sendto.called)
+        self.proactor.sendto.assert_called_with(
+            self.sock, b'data', addr=('0.0.0.0', 1234))
+
+    def test_sendto_no_data(self):
+        transport = self.datagram_transport()
+        transport._buffer.append((b'data', ('0.0.0.0', 12345)))
+        transport.sendto(b'', ())
+        self.assertFalse(self.sock.sendto.called)
+        self.assertEqual(
+            [(b'data', ('0.0.0.0', 12345))], list(transport._buffer))
+
+    def test_sendto_buffer(self):
+        transport = self.datagram_transport()
+        transport._buffer.append((b'data1', ('0.0.0.0', 12345)))
+        transport._write_fut = object()
+        transport.sendto(b'data2', ('0.0.0.0', 12345))
+        self.assertFalse(self.proactor.sendto.called)
+        self.assertEqual(
+            [(b'data1', ('0.0.0.0', 12345)),
+             (b'data2', ('0.0.0.0', 12345))],
+            list(transport._buffer))
+
+    def test_sendto_buffer_bytearray(self):
+        data2 = bytearray(b'data2')
+        transport = self.datagram_transport()
+        transport._buffer.append((b'data1', ('0.0.0.0', 12345)))
+        transport._write_fut = object()
+        transport.sendto(data2, ('0.0.0.0', 12345))
+        self.assertFalse(self.proactor.sendto.called)
+        self.assertEqual(
+            [(b'data1', ('0.0.0.0', 12345)),
+             (b'data2', ('0.0.0.0', 12345))],
+            list(transport._buffer))
+        self.assertIsInstance(transport._buffer[1][0], bytes)
+
+    def test_sendto_buffer_memoryview(self):
+        data2 = memoryview(b'data2')
+        transport = self.datagram_transport()
+        transport._buffer.append((b'data1', ('0.0.0.0', 12345)))
+        transport._write_fut = object()
+        transport.sendto(data2, ('0.0.0.0', 12345))
+        self.assertFalse(self.proactor.sendto.called)
+        self.assertEqual(
+            [(b'data1', ('0.0.0.0', 12345)),
+             (b'data2', ('0.0.0.0', 12345))],
+            list(transport._buffer))
+        self.assertIsInstance(transport._buffer[1][0], bytes)
+
+    @mock.patch('asyncio.proactor_events.logger')
+    def test_sendto_exception(self, m_log):
+        data = b'data'
+        err = self.proactor.sendto.side_effect = RuntimeError()
+
+        transport = self.datagram_transport()
         transport._fatal_error = mock.Mock()
-
-        self.loop.call_exception_handler = mock.Mock()
-        self.protocol.get_buffer.side_effect = LookupError()
-
-        transport._loop_reading()
+        transport.sendto(data, ())
 
         self.assertTrue(transport._fatal_error.called)
-        self.assertTrue(self.protocol.get_buffer.called)
-        self.assertFalse(self.protocol.buffer_updated.called)
+        transport._fatal_error.assert_called_with(
+                                   err,
+                                   'Fatal write error on datagram transport')
+        transport._conn_lost = 1
 
-    def test_get_buffer_zerosized(self):
-        transport = self.socket_transport()
+        transport._address = ('123',)
+        transport.sendto(data)
+        transport.sendto(data)
+        transport.sendto(data)
+        transport.sendto(data)
+        transport.sendto(data)
+        m_log.warning.assert_called_with('socket.sendto() raised exception.')
+
+    def test_sendto_error_received(self):
+        data = b'data'
+
+        self.sock.sendto.side_effect = ConnectionRefusedError
+
+        transport = self.datagram_transport()
         transport._fatal_error = mock.Mock()
+        transport.sendto(data, ())
 
-        self.loop.call_exception_handler = mock.Mock()
-        self.protocol.get_buffer.side_effect = lambda hint: bytearray(0)
+        self.assertEqual(transport._conn_lost, 0)
+        self.assertFalse(transport._fatal_error.called)
 
-        transport._loop_reading()
+    def test_sendto_error_received_connected(self):
+        data = b'data'
 
-        self.assertTrue(transport._fatal_error.called)
-        self.assertTrue(self.protocol.get_buffer.called)
-        self.assertFalse(self.protocol.buffer_updated.called)
+        self.proactor.send.side_effect = ConnectionRefusedError
 
-    def test_proto_type_switch(self):
-        self.protocol = test_utils.make_test_protocol(asyncio.Protocol)
-        tr = self.socket_transport()
-
-        res = asyncio.Future(loop=self.loop)
-        res.set_result(b'data')
-
-        tr = self.socket_transport()
-        tr._read_fut = res
-        tr._loop_reading(res)
-        self.loop._proactor.recv.assert_called_with(self.sock, 32768)
-        self.protocol.data_received.assert_called_with(b'data')
-
-        # switch protocol to a BufferedProtocol
-
-        buf_proto = test_utils.make_test_protocol(asyncio.BufferedProtocol)
-        buf = bytearray(4)
-        buf_proto.get_buffer.side_effect = lambda hint: buf
-
-        tr.set_protocol(buf_proto)
-        test_utils.run_briefly(self.loop)
-        res = asyncio.Future(loop=self.loop)
-        res.set_result(4)
-
-        tr._read_fut = res
-        tr._loop_reading(res)
-        self.loop._proactor.recv_into.assert_called_with(self.sock, buf)
-        buf_proto.buffer_updated.assert_called_with(4)
-
-    @unittest.skip('FIXME: bpo-33694: this test is too close to the '
-                   'implementation and should be refactored or removed')
-    def test_proto_buf_switch(self):
-        tr = self.socket_transport()
-        test_utils.run_briefly(self.loop)
-        self.protocol.get_buffer.assert_called_with(-1)
-
-        # switch protocol to *another* BufferedProtocol
-
-        buf_proto = test_utils.make_test_protocol(asyncio.BufferedProtocol)
-        buf = bytearray(4)
-        buf_proto.get_buffer.side_effect = lambda hint: buf
-        tr._read_fut.done.side_effect = lambda: False
-        tr.set_protocol(buf_proto)
-        self.assertFalse(buf_proto.get_buffer.called)
-        test_utils.run_briefly(self.loop)
-        buf_proto.get_buffer.assert_called_with(-1)
-
-    def test_buffer_updated_error(self):
-        transport = self.socket_transport()
+        transport = self.datagram_transport(address=('0.0.0.0', 1))
         transport._fatal_error = mock.Mock()
+        transport.sendto(data)
 
-        self.loop.call_exception_handler = mock.Mock()
-        self.protocol.buffer_updated.side_effect = LookupError()
+        self.assertFalse(transport._fatal_error.called)
+        self.assertTrue(self.protocol.error_received.called)
 
-        res = asyncio.Future(loop=self.loop)
-        res.set_result(10)
-        transport._read_fut = res
-        transport._loop_reading(res)
+    def test_sendto_str(self):
+        transport = self.datagram_transport()
+        self.assertRaises(TypeError, transport.sendto, 'str', ())
 
-        self.assertTrue(transport._fatal_error.called)
-        self.assertFalse(self.protocol.get_buffer.called)
-        self.assertTrue(self.protocol.buffer_updated.called)
+    def test_sendto_connected_addr(self):
+        transport = self.datagram_transport(address=('0.0.0.0', 1))
+        self.assertRaises(
+            ValueError, transport.sendto, b'str', ('0.0.0.0', 2))
 
-    def test_loop_eof_received_error(self):
-        res = asyncio.Future(loop=self.loop)
-        res.set_result(0)
+    def test_sendto_closing(self):
+        transport = self.datagram_transport(address=(1,))
+        transport.close()
+        self.assertEqual(transport._conn_lost, 1)
+        transport.sendto(b'data', (1,))
+        self.assertEqual(transport._conn_lost, 2)
 
-        self.protocol.eof_received.side_effect = LookupError()
+    def test__loop_writing_closing(self):
+        transport = self.datagram_transport()
+        transport._closing = True
+        transport._loop_writing()
+        self.assertIsNone(transport._write_fut)
+        test_utils.run_briefly(self.loop)
+        self.sock.close.assert_called_with()
+        self.protocol.connection_lost.assert_called_with(None)
 
-        tr = self.socket_transport()
-        tr._fatal_error = mock.Mock()
+    def test__loop_writing_exception(self):
+        err = self.proactor.sendto.side_effect = RuntimeError()
 
-        tr.close = mock.Mock()
-        tr._read_fut = res
-        tr._loop_reading(res)
-        self.assertFalse(self.loop._proactor.recv_into.called)
-        self.assertTrue(self.protocol.eof_received.called)
-        self.assertTrue(tr._fatal_error.called)
+        transport = self.datagram_transport()
+        transport._fatal_error = mock.Mock()
+        transport._buffer.append((b'data', ()))
+        transport._loop_writing()
 
-    def test_loop_reading_data(self):
-        res = asyncio.Future(loop=self.loop)
-        res.set_result(4)
+        transport._fatal_error.assert_called_with(
+                                   err,
+                                   'Fatal write error on datagram transport')
 
-        tr = self.socket_transport()
-        tr._read_fut = res
-        tr._loop_reading(res)
-        self.loop._proactor.recv_into.assert_called_with(self.sock, self.buf)
-        self.protocol.buffer_updated.assert_called_with(4)
+    def test__loop_writing_error_received(self):
+        self.proactor.sendto.side_effect = ConnectionRefusedError
 
-    def test_loop_reading_no_data(self):
-        res = asyncio.Future(loop=self.loop)
-        res.set_result(0)
+        transport = self.datagram_transport()
+        transport._fatal_error = mock.Mock()
+        transport._buffer.append((b'data', ()))
+        transport._loop_writing()
 
-        tr = self.socket_transport()
-        self.assertRaises(AssertionError, tr._loop_reading, res)
+        self.assertFalse(transport._fatal_error.called)
 
-        tr.close = mock.Mock()
-        tr._read_fut = res
-        tr._loop_reading(res)
-        self.assertFalse(self.loop._proactor.recv_into.called)
-        self.assertTrue(self.protocol.eof_received.called)
-        self.assertTrue(tr.close.called)
+    def test__loop_writing_error_received_connection(self):
+        self.proactor.send.side_effect = ConnectionRefusedError
 
-    def test_loop_reading_aborted(self):
-        err = self.loop._proactor.recv_into.side_effect = \
-            ConnectionAbortedError()
+        transport = self.datagram_transport(address=('0.0.0.0', 1))
+        transport._fatal_error = mock.Mock()
+        transport._buffer.append((b'data', ()))
+        transport._loop_writing()
 
-        tr = self.socket_transport()
-        tr._fatal_error = mock.Mock()
-        tr._loop_reading()
-        tr._fatal_error.assert_called_with(
-            err, 'Fatal read error on pipe transport')
+        self.assertFalse(transport._fatal_error.called)
+        self.assertTrue(self.protocol.error_received.called)
 
-    def test_loop_reading_aborted_closing(self):
-        self.loop._proactor.recv.side_effect = ConnectionAbortedError()
-
-        tr = self.socket_transport()
-        tr._closing = True
-        tr._fatal_error = mock.Mock()
-        tr._loop_reading()
-        self.assertFalse(tr._fatal_error.called)
-
-    def test_loop_reading_aborted_is_fatal(self):
-        self.loop._proactor.recv_into.side_effect = ConnectionAbortedError()
-        tr = self.socket_transport()
-        tr._closing = False
-        tr._fatal_error = mock.Mock()
-        tr._loop_reading()
-        self.assertTrue(tr._fatal_error.called)
-
-    def test_loop_reading_conn_reset_lost(self):
-        err = self.loop._proactor.recv_into.side_effect = ConnectionResetError()
-
-        tr = self.socket_transport()
-        tr._closing = False
-        tr._fatal_error = mock.Mock()
-        tr._force_close = mock.Mock()
-        tr._loop_reading()
-        self.assertFalse(tr._fatal_error.called)
-        tr._force_close.assert_called_with(err)
-
-    def test_loop_reading_exception(self):
-        err = self.loop._proactor.recv_into.side_effect = OSError()
-
-        tr = self.socket_transport()
-        tr._fatal_error = mock.Mock()
-        tr._loop_reading()
-        tr._fatal_error.assert_called_with(
-            err, 'Fatal read error on pipe transport')
-
-    def test_pause_resume_reading(self):
-        tr = self.socket_transport()
-        futures = []
-        for msg in [10, 20, 30, 40, 0]:
-            f = asyncio.Future(loop=self.loop)
-            f.set_result(msg)
-            futures.append(f)
-
-        self.loop._proactor.recv_into.side_effect = futures
-        self.loop._run_once()
-        self.assertFalse(tr._paused)
-        self.assertTrue(tr.is_reading())
-        self.loop._run_once()
-        self.protocol.buffer_updated.assert_called_with(10)
-        self.loop._run_once()
-        self.protocol.buffer_updated.assert_called_with(20)
-
-        tr.pause_reading()
-        tr.pause_reading()
-        self.assertTrue(tr._paused)
-        self.assertFalse(tr.is_reading())
-        for i in range(10):
-            self.loop._run_once()
-        self.protocol.buffer_updated.assert_called_with(20)
-
-        tr.resume_reading()
-        tr.resume_reading()
-        self.assertFalse(tr._paused)
-        self.assertTrue(tr.is_reading())
-        self.loop._run_once()
-        self.protocol.buffer_updated.assert_called_with(30)
-        self.loop._run_once()
-        self.protocol.buffer_updated.assert_called_with(40)
-        tr.close()
-
-        self.assertFalse(tr.is_reading())
+    @mock.patch('asyncio.base_events.logger.error')
+    def test_fatal_error_connected(self, m_exc):
+        transport = self.datagram_transport(address=('0.0.0.0', 1))
+        err = ConnectionRefusedError()
+        transport._fatal_error(err)
+        self.assertFalse(self.protocol.error_received.called)
+        m_exc.assert_not_called()
 
 
 class BaseProactorEventLoopTests(test_utils.TestCase):
@@ -733,19 +688,19 @@ class BaseProactorEventLoopTests(test_utils.TestCase):
 
         with mock.patch('asyncio.proactor_events.socket.socketpair',
                         return_value=(self.ssock, self.csock)):
-            self.loop = BaseProactorEventLoop(self.proactor)
+            with mock.patch('signal.set_wakeup_fd'):
+                self.loop = BaseProactorEventLoop(self.proactor)
         self.set_event_loop(self.loop)
 
-    @mock.patch.object(BaseProactorEventLoop, 'call_soon')
     @mock.patch('asyncio.proactor_events.socket.socketpair')
-    def test_ctor(self, socketpair, call_soon):
+    def test_ctor(self, socketpair):
         ssock, csock = socketpair.return_value = (
             mock.Mock(), mock.Mock())
-        loop = BaseProactorEventLoop(self.proactor)
+        with mock.patch('signal.set_wakeup_fd'):
+            loop = BaseProactorEventLoop(self.proactor)
         self.assertIs(loop._ssock, ssock)
         self.assertIs(loop._csock, csock)
         self.assertEqual(loop._internal_fds, 1)
-        call_soon.assert_called_with(loop._loop_self_reading)
         loop.close()
 
     def test_close_self_pipe(self):
@@ -783,6 +738,7 @@ class BaseProactorEventLoopTests(test_utils.TestCase):
 
     def test_loop_self_reading_fut(self):
         fut = mock.Mock()
+        self.loop._self_reading_future = fut
         self.loop._loop_self_reading(fut)
         self.assertTrue(fut.result.called)
         self.proactor.recv.assert_called_with(self.ssock, 4096)
@@ -838,7 +794,7 @@ class BaseProactorEventLoopTests(test_utils.TestCase):
         loop = call_soon.call_args[0][0]
 
         # cancelled
-        fut = asyncio.Future(loop=self.loop)
+        fut = self.loop.create_future()
         fut.cancel()
         loop(fut)
         self.assertTrue(self.sock.close.called)
@@ -860,6 +816,81 @@ class BaseProactorEventLoopTests(test_utils.TestCase):
         self.assertFalse(sock2.close.called)
         self.assertFalse(future2.cancel.called)
 
+    def datagram_transport(self):
+        self.protocol = test_utils.make_test_protocol(asyncio.DatagramProtocol)
+        return self.loop._make_datagram_transport(self.sock, self.protocol)
+
+    def test_make_datagram_transport(self):
+        tr = self.datagram_transport()
+        self.assertIsInstance(tr, _ProactorDatagramTransport)
+        close_transport(tr)
+
+    def test_datagram_loop_writing(self):
+        tr = self.datagram_transport()
+        tr._buffer.appendleft((b'data', ('127.0.0.1', 12068)))
+        tr._loop_writing()
+        self.loop._proactor.sendto.assert_called_with(self.sock, b'data', addr=('127.0.0.1', 12068))
+        self.loop._proactor.sendto.return_value.add_done_callback.\
+            assert_called_with(tr._loop_writing)
+
+        close_transport(tr)
+
+    def test_datagram_loop_reading(self):
+        tr = self.datagram_transport()
+        tr._loop_reading()
+        self.loop._proactor.recvfrom.assert_called_with(self.sock, 256 * 1024)
+        self.assertFalse(self.protocol.datagram_received.called)
+        self.assertFalse(self.protocol.error_received.called)
+        close_transport(tr)
+
+    def test_datagram_loop_reading_data(self):
+        res = self.loop.create_future()
+        res.set_result((b'data', ('127.0.0.1', 12068)))
+
+        tr = self.datagram_transport()
+        tr._read_fut = res
+        tr._loop_reading(res)
+        self.loop._proactor.recvfrom.assert_called_with(self.sock, 256 * 1024)
+        self.protocol.datagram_received.assert_called_with(b'data', ('127.0.0.1', 12068))
+        close_transport(tr)
+
+    @unittest.skipIf(sys.flags.optimize, "Assertions are disabled in optimized mode")
+    def test_datagram_loop_reading_no_data(self):
+        res = self.loop.create_future()
+        res.set_result((b'', ('127.0.0.1', 12068)))
+
+        tr = self.datagram_transport()
+        self.assertRaises(AssertionError, tr._loop_reading, res)
+
+        tr.close = mock.Mock()
+        tr._read_fut = res
+        tr._loop_reading(res)
+        self.assertTrue(self.loop._proactor.recvfrom.called)
+        self.assertFalse(self.protocol.error_received.called)
+        self.assertFalse(tr.close.called)
+        close_transport(tr)
+
+    def test_datagram_loop_reading_aborted(self):
+        err = self.loop._proactor.recvfrom.side_effect = ConnectionAbortedError()
+
+        tr = self.datagram_transport()
+        tr._fatal_error = mock.Mock()
+        tr._protocol.error_received = mock.Mock()
+        tr._loop_reading()
+        tr._protocol.error_received.assert_called_with(err)
+        close_transport(tr)
+
+    def test_datagram_loop_writing_aborted(self):
+        err = self.loop._proactor.sendto.side_effect = ConnectionAbortedError()
+
+        tr = self.datagram_transport()
+        tr._fatal_error = mock.Mock()
+        tr._protocol.error_received = mock.Mock()
+        tr._buffer.appendleft((b'Hello', ('127.0.0.1', 12068)))
+        tr._loop_writing()
+        tr._protocol.error_received.assert_called_with(err)
+        close_transport(tr)
+
 
 @unittest.skipIf(sys.platform != 'win32',
                  'Proactor is supported on Windows only')
@@ -923,7 +954,7 @@ class ProactorEventLoopUnixSockSendfileTests(test_utils.TestCase):
     def prepare(self):
         sock = self.make_socket()
         proto = self.MyProto(self.loop)
-        port = support.find_unused_port()
+        port = socket_helper.find_unused_port()
         srv_sock = self.make_socket(cleanup=False)
         srv_sock.bind(('127.0.0.1', port))
         server = self.run_loop(self.loop.create_server(
@@ -947,7 +978,7 @@ class ProactorEventLoopUnixSockSendfileTests(test_utils.TestCase):
     def test_sock_sendfile_not_a_file(self):
         sock, proto = self.prepare()
         f = object()
-        with self.assertRaisesRegex(events.SendfileNotAvailableError,
+        with self.assertRaisesRegex(asyncio.SendfileNotAvailableError,
                                     "not a regular file"):
             self.run_loop(self.loop._sock_sendfile_native(sock, f,
                                                           0, None))
@@ -956,7 +987,7 @@ class ProactorEventLoopUnixSockSendfileTests(test_utils.TestCase):
     def test_sock_sendfile_iobuffer(self):
         sock, proto = self.prepare()
         f = io.BytesIO()
-        with self.assertRaisesRegex(events.SendfileNotAvailableError,
+        with self.assertRaisesRegex(asyncio.SendfileNotAvailableError,
                                     "not a regular file"):
             self.run_loop(self.loop._sock_sendfile_native(sock, f,
                                                           0, None))
@@ -966,7 +997,7 @@ class ProactorEventLoopUnixSockSendfileTests(test_utils.TestCase):
         sock, proto = self.prepare()
         f = mock.Mock()
         f.fileno.return_value = -1
-        with self.assertRaisesRegex(events.SendfileNotAvailableError,
+        with self.assertRaisesRegex(asyncio.SendfileNotAvailableError,
                                     "not a regular file"):
             self.run_loop(self.loop._sock_sendfile_native(sock, f,
                                                           0, None))
